import logging

import click

from cli.cli import deployment as deployment_cli
from cli.context import Context, pass_context
from cli.deployment.deployment import DeploymentHandler
from cli.utils import Printer

logger = logging.getLogger()


@deployment_cli.command()
@pass_context
def list(context: Context) -> None:
    try:
        client = DeploymentHandler(context)
        items = client.list()
        Printer.print_dict(
            items=items, headers=["component_id", "version"]
        )

    except Exception as e:
        click.secho(f"Error listing database: {str(e)}", fg="red")
        return



from unittest.mock import patch
from cli.constants import *
from cli.engine.datalake import datalake_app
from cli.engine.manager import DatalakeManager
from cli.tests.test_generic import SplightCLITest


class TestDump(SplightCLITest):

    def setUp(self):
        super().setUp()
        self.configure()

    def test_dump(self):
        with patch.object(DatalakeManager, "dump") as mocked_dump:
            result = self.runner.invoke(datalake_app, ['dump', 'collection'], obj=self.context, catch_exceptions=False)
            self.assertEqual(result.output, '')
            mocked_dump.assert_called_once()



import json
from typing import List, Optional

import typer
from rich.console import Console
from splight_models import Graph

from cli.constants import error_style
from cli.engine.manager import ResourceManager, ResourceManagerException

graph_app = typer.Typer(
    name="Splight Engine Graph",
    add_completion=True,
    rich_markup_mode="rich",
    no_args_is_help=True,
)

console = Console()
MODEL = Graph


@graph_app.command()
def list(
    ctx: typer.Context,
    filters: Optional[List[str]] = typer.Option(
        None,
        "--filter",
        "-f",
        help="Query param in the form key=value",
    )
):
    manager = ResourceManager(
        client=ctx.obj.framework.setup.DATABASE_CLIENT(),
        model=MODEL,
    )
    params = manager.get_query_params(filters)
    manager.list(params)


@graph_app.command()
def get(
    ctx: typer.Context,
    instance_id: str = typer.Argument(..., help="The Graph's ID"),
):
    manager = ResourceManager(
        client=ctx.obj.framework.setup.DATABASE_CLIENT(),
        model=MODEL,
    )
    try:
        manager.get(instance_id)
    except ResourceManagerException as exc:
        console.print(exc, style=error_style)


@graph_app.command()
def create(
    ctx: typer.Context,
    path: str = typer.Argument(
        ..., help="Path to JSON file with resource data"
    ),
):
    manager = ResourceManager(
        client=ctx.obj.framework.setup.DATABASE_CLIENT(),
        model=MODEL,
    )
    with open(path, "r") as fid:
        body = json.load(fid)
    manager.create(data=body)


@graph_app.command()
def delete(
    ctx: typer.Context,
    instance_id: str = typer.Argument(
        ..., help="The ID of the instance to be removed"
    ),
):
    manager = ResourceManager(
        client=ctx.obj.framework.setup.DATABASE_CLIENT(),
        model=MODEL,
    )
    manager.delete(instance_id)



from typing import Optional

import typer

from cli.component import component_app
from cli.config import config_app
from cli.context import Context
from cli.engine import engine_app
from cli.hub import hub_app
from cli.version import __version__
from cli.workspace import workspace_app

app = typer.Typer(
    name="Splight Command Line",
    add_completion=True,
    rich_markup_mode="rich",
    no_args_is_help=True,
)

app.add_typer(config_app, name="configure")
app.add_typer(component_app, name="component")
app.add_typer(engine_app, name="engine")
app.add_typer(hub_app, name="hub")
app.add_typer(workspace_app, name="workspace")


def version_callback(version: bool):
    if version:
        print(__version__)
        raise typer.Exit()


@app.callback()
def main(
    ctx: typer.Context,
    version: Optional[bool] = typer.Option(
        None, "--version", "-v", callback=version_callback, is_eager=True
    ),
):
    ctx.obj = Context()


# @cli.group()
# @pass_context
# @needs_credentials
# def component(ctx):
#     ctx.obj = Context()
#
#
# @cli.group()
# @pass_context
# @needs_credentials
# def datalake(ctx):
#     ctx.obj = Context()
#
#
# @cli.group()
# @pass_context
# @needs_credentials
# def database(ctx):
#     ctx.obj = Context()
#
#
# @cli.group()
# @pass_context
# @needs_credentials
# def deployment(ctx):
#     ctx.obj = Context()
#
#
# @cli.group()
# @pass_context
# def workspace(ctx):
#     ctx.obj = Context()
#
#
# @cli.group(cls=DefaultGroup, default="config", default_if_no_args=True)
# @pass_context
# def configure(ctx: Context, from_json: str = False) -> None:
#     ctx.obj = Context()



import os
from jinja2 import Template
from cli.constants import TEMPLATES_FOLDER


class MissingTemplate(Exception):
    pass


def get_template(name) -> Template:
    template_path = os.path.join(TEMPLATES_FOLDER, name)
    if not os.path.exists(template_path):
        raise MissingTemplate(f"Unable to find template {template_path}")
    with open(template_path, "r+") as f:
        content = f.read()
    return Template(content)



import importlib
import os
import sys
import subprocess
from typing import Dict, List, Union, Optional
from pathlib import Path
from splight_lib.component import AbstractComponent
from splight_lib import logging
from cli.component.spec import Spec
from cli.constants import (
    MAIN_CLASS_NAME,
    SPEC_FILE,
    COMPONENT_FILE, 
    SPEC_FILE,
    INIT_FILE,
    README_FILE_1,
    README_FILE_2
)
from cli.utils import get_json_from_file, input_single

logger = logging.getLogger()
Primitive = Union[int, str, float, bool]


class ComponentLoader:
    _MAIN_CLASS_NAME: str = "Main"
    REQUIRED_FILES = [
        COMPONENT_FILE, SPEC_FILE, INIT_FILE, README_FILE_1
    ]

    def __init__(self, path: str) -> None:
        abs_path = str(Path(path).resolve())

        self.base_path = abs_path
        self.component_directory_name = abs_path.split(os.sep)[-1]
        self.module = None
        sys.path.append(os.path.dirname(abs_path))
        self._validate()

    def load(self):
        try:
            self.module = importlib.import_module(self.component_directory_name)
        except Exception as e:
            raise Exception(
                f"Failed importing component {self.component_directory_name}: {str(e)}"
            ) from e
        self._validate()
        self.main_class = getattr(self.module, MAIN_CLASS_NAME)
        return self.main_class

    def _validate(self):
        # VALIDATE FILES
        for required_file in [x for x in self.REQUIRED_FILES if x!=README_FILE_1]:
            if not os.path.isfile(os.path.join(self.base_path, required_file)):
                raise Exception(
                    f"{required_file} file is missing in {self.base_path}"
                )
        # retrocompatibility for components with README without extension
        if not os.path.isfile(os.path.join(self.base_path, README_FILE_1)) \
            and not os.path.isfile(os.path.join(self.base_path, README_FILE_2)):
                raise Exception(
                    f"No {README_FILE_1} or {README_FILE_2} found in {self.base_path}"
                )
        if self.module:
            # VALIDATE MODULE HAS MAIN CLASS
            if not hasattr(self.module, MAIN_CLASS_NAME):
                raise Exception(
                    f"Component does not have a class called {MAIN_CLASS_NAME}"
                )
            # VALIDATE MAIN CLASS IS INHERITING FROM ABSTRACTCOMPONENT
            if not any(
                [
                    parent_class.__name__ == AbstractComponent.__name__
                    for parent_class in getattr(self.module, MAIN_CLASS_NAME).__mro__
                ]
            ):
                raise Exception(
                    f"Component class {MAIN_CLASS_NAME} must inherit from one of Splight's component classes"
                )


class SpecLoader:
    _NAME_KEY: str = "name"
    _VERSION_KEY: str = "version"
    _SPLIGHT_CLI_VERSION_KEY: str = "splight_cli_version"
    _PRIVACY_POLICY_KEY: str = "privacy_policy"
    _COMPONENT_KEY: str = "component_type"
    _CT_KEY: str = "custom_types"
    _INPUT_KEY: str = "input"
    _OUTPUT_KEY: str = "output"
    _BINDINGS_KEY: str = "bindings"
    _COMMANDS_KEY: str = "commands"
    _ENDPOINTS_KEY: str = "endpoints"

    def __init__(self, path: str):
        self.raw_spec = get_json_from_file(os.path.join(path, SPEC_FILE))
        self._validate()

    def load(self, input_parameters: Optional[List[Dict]] = None, prompt_input = True):
        input_parameters = input_parameters if input_parameters else self.raw_spec['input']
        if prompt_input:
            input_parameters = self._load_or_prompt_input(input_parameters)
        self.raw_spec["input"] = input_parameters
        self._validate()
        return Spec.parse_obj(self.raw_spec)

    def _load_or_prompt_input(
        self,
        input_parameters: List[Dict],
        prefix: str = "",
    ):
        for param in input_parameters:
            value = param.get("value")
            if value is None:
                new_value = self._prompt_param(param, prefix=prefix)
                param["value"] = new_value

        return input_parameters

    def _prompt_param(
        self, param: Dict, prefix: str = ""
    ) -> Primitive:
        param_name = param["name"]
        new_value = input_single(
            {
                "name": f"{prefix}.{param_name}",
                "type": param["type"],
                "required": param.get("required", True),
                "multiple": param.get("multiple", False),
                "value": param.get("value"),
            }
        )
        return new_value

    def _validate(self):
        Spec.verify(self.raw_spec)


class InitLoader:
    def __init__(self, path: str) -> None:
        self.path = path
        self._validate()

    def _handle_RUN(self, command: List[str]) -> None:
        command: str = " ".join(command)
        logger.debug(f"Running initialization command: {command} ...")
        try:
            subprocess.run(command, check=True, cwd=self.path, shell=True)
        except subprocess.CalledProcessError as e:
            raise Exception(
                f"Failed to run command: {e.cmd}. Output: {e.output}"
            )

    def _load_commands(self) -> List[List[str]]:
        file = os.path.join(self.path, INIT_FILE)
        lines: List[str] = []
        lines.append(["RUN", "pip", "cache", "purge"])
        with open(file) as f:
            for line in f:
                line = line.strip()
                if line.startswith("#") or line == "":
                    continue
                lines.append(line.split(" "))
        return lines

    def load(self):
        commands = self._load_commands()
        for command in commands:
            prefix: str = command[0]
            handler = getattr(self, f'_handle_{prefix}', None)
            if not handler:
                raise Exception(f"Invalid command: {prefix}")
            handler(command[1:])

    def _validate(self):
        # VALIDATE FILES
        if not os.path.isfile(os.path.join(self.path, INIT_FILE)):
            raise Exception(
                f"{INIT_FILE} file is missing in {self.path}"
            )



import typer

from cli.context import check_credentials
from cli.hub.component import component_app

hub_app = typer.Typer(
    name="Splight Hub",
    add_completion=True,
    rich_markup_mode="rich",
    no_args_is_help=True,
)

hub_app.add_typer(component_app, name="component")


@hub_app.callback(invoke_without_command=True)
def hub_callback(ctx: typer.Context):
    check_credentials(ctx)



class InvalidSplightCLIVersion(Exception):
    def __init__(self, component_cli_version: str, cli_version: str):
        self._msg = (
            f"Component uses splight cli version {component_cli_version} but "
            f"should be {cli_version}"
        )

    def __str__(self) -> str:
        return self._msg

class ReadmeExists(Exception):
    def __init__(self, readme_path: str):
        self._msg = (
            f"\nReadme already exists at {readme_path}"
            f"\nRemove it or use --force to overwrite it"
        )

    def __str__(self) -> str:
        return self._msg


import os
from unittest import TestCase
# from click.testing import CliRunner
from typer.testing import CliRunner
from splight_lib.settings import setup

from cli.component.spec import Spec
from cli.constants import DEFAULT_WORKSPACE, SPEC_FILE
from cli.context import Context
from cli.settings import SplightCLISettings
from cli.utils import get_json_from_file


class FakeFramework:
    @property
    def setup(self):
        return setup

    def configure(self, environment):
        # empty_keys = [
        #     key for key, value in environment.items() if value is None
        # ]
        setup.configure(environment)


class FakeWorkspace:
    _current_workspace = "test"
    _workspaces = {"test": DEFAULT_WORKSPACE}

    def __init__(self):
        self._current_settings = SplightCLISettings.parse_obj(self._workspaces[self._current_workspace])

    @property
    def settings(self):
        return self._current_settings

    @property
    def current_workspace(self):
        return self._current_workspace

    def update_workspace(self, new_settings):
        self._current_settings = SplightCLISettings.parse_obj(new_settings.dict())

    def select_workspace(self, value):
        self._current_workspace = value

    def list_workspaces(self):
        workspaces = self._workspaces.keys()
        return [f"{key}*" if key == self._current_workspace else f"{key}" for key in workspaces]

    def delete_workspace(self, workspace_name):
        if workspace_name not in self._workspaces:
            raise Exception('Not a valid namespace name')
        del self._workspaces[workspace_name]

    def create_workspace(self, workspace_name):
        if workspace_name in self._workspaces:
            raise Exception('Name already exists')
        self._workspaces.update({
            workspace_name: DEFAULT_WORKSPACE
        })
        self._current_workspace = workspace_name


class FakeContext(Context):
    def __init__(self):
        self.__workspace = FakeWorkspace()
        self.__framework = FakeFramework()

    @property
    def framework(self):
        self.__framework.configure(self.workspace.settings.dict())
        return self.__framework

    @property
    def workspace(self):
        return self.__workspace

    def __repr__(self):
        return f"<Context {self.__workspace}>"


class SplightCLITest(TestCase):

    def setUp(self):
        self.path = os.path.join(os.path.dirname(__file__), "TestHub")
        self.component_json = get_json_from_file(os.path.join(self.path, SPEC_FILE))
        self.spec = Spec.parse_obj(self.component_json)
        self.name = self.component_json['name']
        self.version = self.component_json['version']
        self.splight_cli_version = self.component_json['splight_cli_version']
        self.component_type = self.component_json['component_type']
        self.privacy_policy = self.component_json['privacy_policy']
        self.component_id = self.component_json["component_id"]
        self.custom_types = self.component_json['custom_types']
        self.input = self.component_json['input']
        self.output = self.component_json['output']
        self.bindings = self.component_json["bindings"]
        self.tags = self.component_json['tags']
        self.commands = self.component_json['commands']
        self.endpoints = self.component_json['endpoints']
        self.context = FakeContext()
        self.runner = CliRunner()
        self.custompath = self.name + self.version + self.component_id

    @property
    def default_configuration(self):
        return SplightCLISettings.parse_obj({
            "SPLIGHT_ACCESS_ID": "access_id",
            "SPLIGHT_SECRET_KEY": "secret_key",
            "DATABASE_CLIENT": "fake_splight_lib.database.FakeDatabaseClient",
            "HUB_CLIENT": "fake_splight_lib.hub.FakeHubClient",
            "DATALAKE_CLIENT": "fake_splight_lib.datalake.FakeDatalakeClient",
            "DEPLOYMENT_CLIENT": "fake_splight_lib.deployment.FakeDeploymentClient",
        })

    def configure(self):
        self.context.workspace.update_workspace(self.default_configuration)

    def tearDown(self) -> None:
        return super().tearDown()



import os
from pathlib import Path
from pydantic import AnyUrl
from rich.style import Style

from cli.settings import SplightCLISettings

error_style = Style(color="red", bold=True)
success_style = Style(color="green")
warning_style = Style(color="yellow")

SPLIGHT_PATH = os.path.join(os.path.expanduser("~"), '.splight')
BASE_DIR = Path(__file__).resolve().parent.parent
TEMPLATES_FOLDER = os.path.join(BASE_DIR, "cli", "component", "templates")

COMPRESSION_TYPE = "7z"
DEFAULT_NAMESPACE = 'NO_NAMESPACE'
DEFAULT_COMPONENT_ID = "DEMO"
DEFAULT_WORKSPACE_NAME = 'default'
DEFAULT_WORKSPACE = SplightCLISettings().dict()
DEFAULT_WORKSPACES = {
    DEFAULT_WORKSPACE_NAME: DEFAULT_WORKSPACE
}

CONFIG_FILE = os.path.join(SPLIGHT_PATH, 'config')
COMPONENT_FILE = "__init__.py"
SPEC_FILE = "spec.json"
INIT_FILE = "Initialization"
README_FILE_1 = "README.md" 
README_FILE_2 = "README" 
MAIN_CLASS_NAME = "Main"

VALID_PARAMETER_VALUES = {
    "int": int,
    "bool": bool,
    "str": str,
    "float": float,
    "url": AnyUrl,
    "datetime": None,
    "file": None,  # UUID
    "File": None,  # UUID
    "Asset": None,  # UUID,
    "Attribute": None,  # UUID,
    "Component": None,  # UUID,
    "Graph": None,  # UUID,
    "Query": None, # UUID,
    "Mapping": None, # UUID
}

VALID_DEPENDS_ON = [
    ("Asset", "Graph"),
    ("Attribute", "Asset"),
]



from splight_lib.component import AbstractComponent


class Main(AbstractComponent):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

    def test(self):
        print("HELLO2")

    def start(self):
        print("HELLO")
        self.test()
        self.terminate()

    def hello(self, message: str):
        return "hello" + message


import re
from unittest.mock import patch

# from cli.hub.component import component_app
from cli.hub import hub_app
from cli.hub.component.hub_manager import HubComponentManager

# from cli.component.handler import ComponentHandler
from cli.tests.test_generic import SplightCLITest


class TestConfigure(SplightCLITest):
    def test_configure_requested(self):
        with patch.object(
            HubComponentManager, "list_components", return_value=None
        ):
            result = self.runner.invoke(
                hub_app,
                ["component", "list"],
                obj=self.context,
                catch_exceptions=False,
            )
        ansi_escape = re.compile(r"\x1B(?:[@-Z\\-_]|\[[0-?]*[ -/]*[@-~])")
        output = ansi_escape.sub("", result.output)
        self.assertEqual(result.exit_code, 1)
        self.assertIn("splight configure", output)

    def test_configure_not_requested(self):
        self.configure()
        with patch.object(
            HubComponentManager, "list_components", return_value=None
        ):
            result = self.runner.invoke(
                hub_app,
                ["component", "list"],
                obj=self.context,
                catch_exceptions=False,
            )
        self.assertEqual(result.exit_code, 0)



import os
import uuid
from typing import Dict, List, Optional

from jinja2 import Template
from splight_lib.execution import Thread
from rich.console import Console

from cli.component.loaders import ComponentLoader, InitLoader, SpecLoader
from cli.component.spec import Spec
from cli.component.exceptions import (
    InvalidSplightCLIVersion,
    ReadmeExists
)
from cli.constants import (
    COMPONENT_FILE,
    README_FILE_1
)
from cli.utils import get_template, input_single
from cli.version import __version__
from splight_models import Component as ComponentModel


console = Console()

class Component:
    name = None
    version = None

    def __init__(self, context):
        self.context = context

    def create(self, name: str, version: str = "0.1.0", component_path: str = ""):
        Spec.verify(
            {
                "name": name,
                "version": version,
                "splight_cli_version": __version__,
                "custom_types": [],
                "input": [],
                "output": [],
                "endpoints": [],
            }
        )

        if component_path and not os.path.exists(component_path):
            component_path = os.path.join(f"{component_path}")
            os.makedirs(component_path)

        for file_name in ComponentLoader.REQUIRED_FILES:
            template_name = file_name
            file_path = os.path.join(component_path, file_name)
            component_id = str(uuid.uuid4())
            if file_name == COMPONENT_FILE:
                template_name = "component.py"
            template: Template = get_template(template_name)
            file = template.render(
                component_name=name,
                version=version,
                component_id=component_id,
                splight_cli_version=__version__,
            )
            with open(file_path, "w+") as f:
                f.write(file)

    def run(
        self,
        path: str,
        input_parameters: Optional[List[Dict]] = None,
        component_id: str = None,
    ):
        # Load py module and validate Splight Component structure
        loader = ComponentLoader(path=path)
        component_class = loader.load()
        # Load json and validate Spec structure
        loader = SpecLoader(path=path)

        if component_id and not input_parameters:
            remote_input_parameters = []
            db_client = self.context.framework.setup.DATABASE_CLIENT()
            component_input = db_client.get(
                ComponentModel,
                id=component_id,
                first=True
            ).input

            for input in component_input:
                remote_input_parameters.append(input.__dict__)

            input_parameters = remote_input_parameters

        run_spec = loader.load(input_parameters=input_parameters)
        self._validate_cli_version(run_spec.splight_cli_version)
        component = component_class(
            run_spec=run_spec.dict(),
            initial_setup=self.context.workspace.settings.dict(),
            component_id=component_id,
        )
        component.execution_client.start(Thread(target=component.start))

    def upgrade(self, from_component_id: str, to_component_id: str):
        db_client = self.context.framework.setup.DATABASE_CLIENT()
        from_component = db_client.get(
            ComponentModel,
            id=from_component_id,
            first=True
        )
        to_component = db_client.get(
            ComponentModel,
            id=to_component_id,
            first=True
        )

        from_inputs = from_component.input
        to_inputs = to_component.input
        for param in to_inputs:
            has_value = False

            for old in from_inputs:
                if param.name == old.name:
                    param.value = old.value
                    has_value = True
                    break

            if not has_value:
                param.value = input_single(
                    {
                        "name": param.name,
                        "type": param.type,
                        "required": param.required,
                        "multiple": param.multiple,
                        "value": param.value,
                    }
                )

        to_component.input = to_inputs

        # TODO: also upgrade component objects

        db_client.save(instance=to_component)

    def install_requirements(self, path: str):
        loader = InitLoader(path=path)
        loader.load()

    def _validate_cli_version(self, component_cli_version: str):
        if component_cli_version != __version__:
            raise InvalidSplightCLIVersion(component_cli_version, __version__)

    def readme(self, path: str, force: Optional[bool] = False):
        loader = SpecLoader(path=path)
        spec = loader.load().dict()
        name, version = spec["name"], spec["version"]
        if os.path.exists(os.path.join(path, README_FILE_1)):
            if not force:
                raise ReadmeExists(path)
            else:
                os.remove(os.path.join(path, README_FILE_1))
        template = get_template("auto_readme.md")
        readme = template.render(
            component_name=name,
            version=version,
            component_type=spec.get("component_type",""),
            inputs=spec.get("input", []),
            bindings=spec.get("bindings",[]),
            output=spec.get("output", []),
        )
        with open(os.path.join(path, README_FILE_1), "w+") as f:
            f.write(readme)
        console.print(f"{README_FILE_1} created for {name} {version}")



from typing import Dict

from cli.settings import SplightCLISettings


def extract_headers(config: SplightCLISettings) -> Dict[str, str]:
    header = f"Splight {config.SPLIGHT_ACCESS_ID} {config.SPLIGHT_SECRET_KEY}"
    return {"Authorization": header}



import os
import json

def get_json_from_file(file_path: str):
    if not os.path.isfile(file_path):
        raise Exception(f"String provided: \"{file_path}\" is not a file, please provide a filepath or use -fs flag")
    with open(file_path, 'r') as f:
        try:
            return json.load(f)
        except:
            raise Exception("File is not a valid JSON")


import os
import yaml


def get_yaml_from_file(file_path: str):
    if not os.path.isfile(file_path):
        raise Exception(f"String provided: \"{file_path}\" is not a file, please provide a filepath or use -fs flag")
    with open(file_path, 'r') as f:
        try:
            data = yaml.load(f, Loader=yaml.FullLoader)
            return data if data is not None else {}
        except yaml.YAMLError as e:
            raise Exception("File is not a valid YAML") from e


def save_yaml_to_file(payload: str, file_path: str):
    with open(file_path, 'w+') as f:
        yaml.Dumper.ignore_aliases = lambda *args : True  # TOO RISKY to add refs on repetitions
        yaml.dump(payload, f)



import json
import os
import shutil
from typing import Any, Dict, Optional

import py7zr
from rich.console import Console
from rich.table import Table
from splight_abstract.hub import AbstractHubClient
from splight_models import HubComponent, HubComponentVersion
from splight_models.constants import ComponentType

from cli.constants import (
    COMPRESSION_TYPE,
    README_FILE_1,
    README_FILE_2,
    SPEC_FILE,
    success_style,
)
from cli.hub.component.exceptions import (
    ComponentAlreadyExists,
    ComponentPullError,
    ComponentPushError,
    ComponentDirectoryAlreadyExists,
)
from cli.utils.loader import Loader

console = Console()


class HubComponentManager:
    def __init__(self, client: AbstractHubClient):
        self._client = client

    def push(self, path: str, force: Optional[bool] = False):
        with open(os.path.join(path, SPEC_FILE)) as fid:
            spec = json.load(fid)

        name = spec["name"]
        version = spec["version"]

        if not force and self._exists_in_hub(name, version):
            raise ComponentAlreadyExists(name, version)

        with Loader("Pushing Component to Splight Hub"):
            self._upload_component(spec, path)

        console.print("Component pushed succesfully", style=success_style)

    def pull(self, name: str, version: str):
        with Loader("Pulling component from Splight Hub"):
            response = self._client.download(
                data={"name": name, "version": version}
            )

            # TODO: search for a better approach
            version_modified = version.replace(".", "_")
            component_path = f"{name}/{version_modified}"
            versioned_name = f"{name}-{version}"
            file_name = f"{versioned_name}.{COMPRESSION_TYPE}"
            if os.path.exists(component_path):
                raise ComponentDirectoryAlreadyExists(component_path)

            try:
                with open(file_name, "wb") as fid:
                    fid.write(response[0])

                with py7zr.SevenZipFile(file_name, "r") as z:
                    z.extractall(path=".")
                shutil.move(f"{versioned_name}", component_path)
            except Exception as exc:
                raise ComponentPullError(name, version) from exc
            finally:
                if os.path.exists(file_name):
                    os.remove(file_name)

    def list_components(self):
        components = self._client.mine.get(HubComponent)
        table = Table("Name")
        for item in components:
            table.add_row(item.name)
        console.print(table)

    def versions(self, name: str):
        components = self._client.mine.get(HubComponentVersion, name=name)

        table = Table("Name", "Version", "Verification", "Privacy Policy")
        for item in components:
            table.add_row(
                name, item.version, item.verification, item.privacy_policy
            )
        console.print(table)

    def _upload_component(self, spec: Dict[str, Any], path: str):
        name = spec["name"]
        version = spec["version"]

        file_name = f"{name}-{version}.{COMPRESSION_TYPE}"
        versioned_name = f"{name}-{version}"
        readme_path = os.path.join(path, README_FILE_1)
        if not os.path.exists(readme_path):
            readme_path = os.path.join(path, README_FILE_2)
        try:
            with py7zr.SevenZipFile(file_name, "w") as fid:
                fid.writeall(path, versioned_name)

            data = {
                "name": name,
                "version": version,
                "splight_cli_version": spec["splight_cli_version"],
                "privacy_policy": spec.get("privacy_policy", "private"),
                "tags": spec.get("tags", []),
                "custom_types": json.dumps(spec.get("custom_types", [])),
                "input": json.dumps(spec.get("input", [])),
                "output": json.dumps(spec.get("output", [])),
                "component_type": spec.get("component_type", ComponentType.CONNECTOR.value),
                "commands": json.dumps(spec.get("commands", [])),
                "bindings": json.dumps(spec.get("bindings", [])),
                "endpoints": json.dumps(spec.get("endpoints", [])),
            }
            files = {
                "file": open(file_name, "rb"),
                "readme": open(readme_path, "rb"),
            }
            self._client.upload(
                data=data,
                files=files,
            )
        except Exception as exc:
            raise ComponentPushError(name, version) from exc
        finally:
            if os.path.exists(file_name):
                os.remove(file_name)

    def _exists_in_hub(self, name: str, version: str) -> bool:
        exists = False

        public = self._client.public.get(
            HubComponent, name=name, version=version
        )
        private = self._client.private.get(
            HubComponent, name=name, version=version
        )
        if any([list(public), list(private)]):
            exists = True
        return exists



import json
import re
from unittest.mock import patch

from cli.component import component_app
from cli.component.component import Component
from cli.tests.test_generic import SplightCLITest


class TestRun(SplightCLITest):

    @patch.object(Component, "_validate_cli_version", return_value=None)
    def test_run(self, mock):
        self.component = Component(self.context)
        self.configure()
        result = self.runner.invoke(
            component_app,
            ["run", self.path, "--input", json.dumps(self.input)],
            obj=self.context,
            catch_exceptions=False,
        )
        # Remove ANSI characters that prints with color
        mock.assert_called_once()
        ansi_escape = re.compile(r"\x1B(?:[@-Z\\-_]|\[[0-?]*[ -/]*[@-~])")
        output = ansi_escape.sub("", result.output)
        self.assertEqual(
            output,
            "Running component...\nHELLO\nHELLO2\n",
        )



from unittest.mock import patch
from cli.constants import *
from cli.engine.datalake import datalake_app
from cli.engine.manager import DatalakeManager
from cli.tests.test_generic import SplightCLITest


class TestLoad(SplightCLITest):

    def setUp(self):
        super().setUp()
        self.expected_load_result = "algo"
        self.configure()

    def test_load(self):
        with patch.object(DatalakeManager, "load", return_value=self.expected_load_result):
            result = self.runner.invoke(datalake_app, ['load', '-p', './dump.csv'], obj=self.context, catch_exceptions=False)
            _ = result.output



import typer

from cli.context import check_credentials
from cli.engine.alert import alert_app
from cli.engine.asset import asset_app
from cli.engine.attribute import attribute_app
from cli.engine.component import component_app
from cli.engine.graph import graph_app
from cli.engine.query import query_app
from cli.engine.file import file_app
from cli.engine.secret import secret_app
from cli.engine.setpoint import setpoint_app
from cli.engine.datalake import datalake_app

engine_app = typer.Typer(
    name="Splight Engine",
    add_completion=True,
    rich_markup_mode="rich",
    no_args_is_help=True,
)

engine_app.add_typer(alert_app, name="alert")
engine_app.add_typer(asset_app, name="asset")
engine_app.add_typer(attribute_app, name="attribute")
engine_app.add_typer(component_app, name="component")
engine_app.add_typer(graph_app, name="graph")
engine_app.add_typer(query_app, name="query")
engine_app.add_typer(file_app, name="file")
engine_app.add_typer(secret_app, name="secret")
engine_app.add_typer(setpoint_app, name="setpoint")
engine_app.add_typer(datalake_app, name="datalake")


@engine_app.callback(invoke_without_command=True)
def engine_callback(ctx: typer.Context):
    check_credentials(ctx)



from typing import Dict

from pydantic import BaseModel, BaseSettings, Extra

from cli.version import __version__


class SplightCLISettings(BaseSettings):
    SPLIGHT_ACCESS_ID: str = ""
    SPLIGHT_SECRET_KEY: str = ""
    SPLIGHT_PLATFORM_API_HOST: str = "https://api.splight-ai.com"
    # FRAMEWORK
    DATABASE_CLIENT: str = "remote_splight_lib.database.DatabaseClient"
    DATALAKE_CLIENT: str = "remote_splight_lib.datalake.DatalakeClient"
    HUB_CLIENT: str = "remote_splight_lib.hub.SplightHubClient"
    STORAGE_CLIENT: str = "remote_splight_lib.storage.StorageClient"
    BLOCKCHAIN_CLIENT: str = "remote_splight_lib.blockchain.BlockchainClient"
    DEPLOYMENT_CLIENT: str = "remote_splight_lib.deployment.DeploymentClient"
    COMMUNICATION_CLIENT: str = (
        "remote_splight_lib.communication.CommunicationClient"
    )
    NAMESPACE: str = "NO_NAMESPACE"

    class Config:
        extra = Extra.ignore


class SplightCLIConfig(BaseModel):
    current_workspace: str
    workspaces: Dict[str, SplightCLISettings]


ALL_CONFIG_VARS = SplightCLISettings.__fields__
CONFIG_VARS = [
    "SPLIGHT_ACCESS_ID",
    "SPLIGHT_SECRET_KEY",
]
SPLIGHT_CLI_VERSION = __version__



from enum import Enum
from pydantic import validator, Field
from typing import List, Dict, Optional
from cli.utils import *
from cli.constants import *
from splight_models import (
    Parameter as ModelParameter,
    InputParameter as ModelInputParameter,
    OutputParameter as ModelOutputParameter,
    CommandParameter as ModelCommandParameter,
    CustomType as ModelCustomType,
    Output as ModelOutput,
    Deployment as ModelDeployment,
    Endpoint as ModelEndpoint
)
from splight_models.constants import ComponentType


class ChoiceMixin:
    @validator("choices", check_fields=False)
    def validate_choices(cls, v, values):
        if "type" not in values:
            return v

        type_ = values["type"]

        if type not in ["str", "int", "float"]:
            raise ValueError("Choices can only be used with string, int or float types")

        # choises type match type field
        try:
            class_type = VALID_PARAMETER_VALUES[type_]
            [isinstance(c, class_type) for c in v]
        except Exception as exc:
            raise ValueError(
                f"choices must be list of type {str(class_type)}"
            ) from exc

        return v


class Parameter(ModelInputParameter, ChoiceMixin):
    @validator("value", check_fields=False)
    def validate_value(cls, v, values, field):
        if not set(values.keys()).issuperset({"type", "required", "multiple", "choices"}):
            return v

        type_ = values["type"]

        # allow custom type
        if type_ not in VALID_PARAMETER_VALUES:
            return v

        # UUIDs and Date must be None
        if VALID_PARAMETER_VALUES[type_] is None:
            return v
        
        # if values["required"] and v is None:
        #     raise ValueError("value must be set")

        # if values["multiple"]:
        #     if not isinstance(v, list):
        #         raise ValueError("value must be a list")

        # Make all parameter values a list
        list_v = v if isinstance(v, list) else [v]

        # value type match type field
        try:
            class_type = VALID_PARAMETER_VALUES[type_]
            [isinstance(v_, class_type) for v_ in list_v]
        except Exception as exc:
            raise ValueError(
                f"value must be of type {str(class_type)}"
            ) from exc

        if values["choices"] is not None and not all([v_ in values["choices"] for v_ in list_v]):
            raise ValueError(f"value must be one of {values['choices']}")

        return v


class OutputParameter(ModelOutputParameter, ChoiceMixin):
    @validator("type", check_fields=False)
    def validate_type(cls, type):
        if type not in VALID_PARAMETER_VALUES:
            raise ValueError(f"invalid output type {type}, can not be custom type")
        return type


class CommandParameter(ModelCommandParameter, ChoiceMixin):
    @validator("type", check_fields=False)
    def validate_type(cls, type):
        if type not in VALID_PARAMETER_VALUES:
            raise ValueError(f"invalid output type {type}, can not be custom type")
        return type


class FieldMixin:
    @validator("fields")
    def validate_fields(cls, fields):
        if len(fields) == 0:
            raise ValueError("fields must not be empty")

        _check_unique_names(fields, "fields")
        _check_parameter_depends_on(fields)

        return fields


class CustomType(ModelCustomType, FieldMixin):
    fields: List[ModelParameter]


class Output(ModelOutput, FieldMixin):
    fields: List[OutputParameter]


class Endpoint(ModelEndpoint):
    name: str
    port: int


class PrivacyPolicy(str, Enum):
    PUBLIC = "public"
    PRIVATE = "private"

    

class Spec(ModelDeployment):
    splight_cli_version: str = Field(regex="^(\d+\.)?(\d+\.)?(\*|\d+)$")
    privacy_policy: PrivacyPolicy = PrivacyPolicy.PUBLIC
    tags: List[str] = []
    component_type: Optional[ComponentType] = ComponentType.CONNECTOR
    custom_types: List[CustomType] = []
    input: List[Parameter] = []
    output: List[Output] = []
    endpoints: List[Endpoint] = []

    @validator("name", check_fields=False)
    def validate_name(cls, name):
        invalid_characters = ["/", "-"]
        if not name[0].isupper():
            raise ValueError("value's first letter must be capitalized")
        if any(x in name for x in invalid_characters):
            raise Exception(f"name cannot contain any of these characters: {invalid_characters}")
        return name

    @validator("version", check_fields=False)
    def validate_version(cls, version):
        invalid_characters = ["/", "-"]
        if len(version) > 20:
            raise Exception("value must be 20 characters maximum")

        if any(x in version for x in invalid_characters):
            raise Exception(f"version cannot contain any of these characters: {invalid_characters}")
        return version

    @validator("tags")
    def validate_tags(cls, tags):
        tag_names = set()
        for tag in tags:
            if tag in tag_names:
                raise Exception(f"Tag name {tag} is not unique")
            tag_names.add(tag)
        return tags

    @validator("custom_types")
    def validate_custom_types(cls, custom_types):
        if custom_types is None:
            return custom_types

        _check_unique_names(custom_types, "custom_types")

        valid_types: List[str] = VALID_PARAMETER_VALUES.keys()
        custom_type_names: List[str] = []

        for custom_type in custom_types:
            for field in custom_type.fields:
                if field.type not in valid_types and field.type not in custom_type_names:
                    raise ValueError(f"custom_type {field.type} not defined")
                if field.name in CustomType._reserved_names:
                    raise ValueError(f"custom_type {field.name} not allowed. Reserved names are: {CustomType._reserved_names}")
            custom_type_names.append(custom_type.name)

        return custom_types

    @validator("input")
    def validate_parameters(cls, v, values, field):
        if "custom_types" not in values:
            return values

        _check_unique_names(v, "input parameters")

        custom_type_names: List[str] = [c.name for c in values["custom_types"]]
        valid_types_names: List[str] = list(VALID_PARAMETER_VALUES.keys()) + custom_type_names

        for parameter in v:
            if parameter.type not in valid_types_names:
                raise ValueError(f"input type {parameter.type} not defined")

        # depends on
        _check_parameter_depends_on(v)
        return v

    @validator("output")
    def validate_output(cls, v, values, field):
        _check_unique_names(v, "output parameters")
        return v
    
    @classmethod
    def verify(cls, dict: dict):
        cls(**dict)


def _check_parameter_depends_on(parameters: List[Parameter]):
    parameter_map: Dict[str, Parameter] = {p.name: p for p in parameters}
    for parameter in parameters:
        if not parameter.depends_on:
            continue

        if parameter.depends_on not in parameter_map:
            raise ValueError(f"depends_on must be one of {parameter_map.keys()}")

        depend_parameter = parameter_map[parameter.depends_on]

        if (parameter.type, depend_parameter.type) not in VALID_DEPENDS_ON:
            raise ValueError(f"incompatible dependance: type {parameter.type} can not"
                             f"depend on type {depend_parameter.type}")


def _check_unique_names(parameters: List[Parameter], type: str):
    parameters_names = [p.name for p in parameters]
    if len(parameters) != len(set(parameters_names)):
        raise ValueError(f"{type} must have unique names")



from functools import partial
from pydoc import locate
from typing import List, Type, Union

import click


def list_of(input_value: Union[List, str], param_type: Type = str):
    if isinstance(input_value, list):
        return input_value
    return [param_type(v.strip()) for v in input_value.split(",")]


def input_single(param: dict):
    default = (
        "None"
        if param.get("value") is None and not param["required"]
        else param["value"]
    )
    name = (
        f"{'*' if param['required'] else ' '}{param['name']}: {param['type']}"
    )
    param_type = locate(param["type"])
    if not param_type:
        param_type = str
    value_proc = None
    if param.get("multiple"):
        value_proc = partial(list_of, param_type=param_type)

    val = click.prompt(
        name,
        type=param_type,
        default=default,
        show_default=True,
        value_proc=value_proc,
    )
    if isinstance(val, str):
        val = val.strip(" ")
    if val == "None":
        val = None
    return val



import sys
import json

import typer

from cli.settings import SplightCLISettings, CONFIG_VARS

config_app = typer.Typer(
    name="Splight CLI Configure",
    add_completion=True,
)


@config_app.callback(invoke_without_command=True)
def config(
    ctx: typer.Context,
    from_json: str = typer.Option(
        None,
        "--from-json",
        help="Configuration by json instaed of prompt")
):
    if ctx.invoked_subcommand:
        return

    cli_context = ctx.obj
    try:
        if from_json:
            from_json = json.loads(from_json)
            new_settings = SplightCLISettings.parse_obj(from_json)
        else:
            new_settings_data = {}
            for config_var in CONFIG_VARS:
                default = getattr(
                    cli_context.workspace.settings, config_var, None
                )
                value = typer.prompt(
                    typer.style(config_var, fg='yellow'),
                    type=str,
                    default=default,
                    show_default=True
                )
                new_settings_data.update({config_var: value})
            new_settings = SplightCLISettings.parse_obj(new_settings_data)
        cli_context.workspace.update_workspace(new_settings)
        typer.echo("Configuration saved successfully", color="green")
    except Exception as e:
        typer.echo(f"Error configuring Splight CLI: {str(e)}", color="red")
        sys.exit(1)


@config_app.command(name="get")
def get_variable(
    ctx: typer.Context,
    var_name: str = typer.Argument(..., help="The variable name to get value")
):
    """Prints the value of the requested variable.

    Parameters
    ----------
    ctx: Context
        The current context
    var_name:
        The variable name to get the value
    """
    variable_name = var_name.upper()
    settings = ctx.obj.workspace.settings

    if hasattr(settings, variable_name):
        value = getattr(settings, variable_name)
        typer.echo(value)


@config_app.command(name="set")
def set_variable(
    ctx: typer.Context,
    var_name: str = typer.Argument(..., help="The variable name to update"),
    value: str = typer.Argument(..., help="The new value")
):
    """Sets a value of a variable in the current workspace.

    Parameters
    ----------
    ctx: Context
        The current context
    var_name: str
        The variable's name to be updated
    value: Any
        The new value
    """
    variable_name = var_name.upper()
    settings = ctx.obj.workspace.settings

    if hasattr(settings, variable_name):
        setattr(settings, variable_name, value)
        ctx.obj.workspace.update_workspace(settings)



import json
import logging
from typing import List, Optional
from pathlib import Path

import typer
from rich.console import Console

from cli.context import check_credentials
from cli.component.component import Component
from cli.constants import error_style, success_style

component_app = typer.Typer(
    name="Splight Component",
    add_completion=True,
    rich_markup_mode="rich",
    no_args_is_help=True,
)

logger = logging.getLogger()
console = Console()


@component_app.callback(invoke_without_command=True)
def callback(ctx: typer.Context):
    check_credentials(ctx)


@component_app.command()
def create(
    ctx: typer.Context,
    name: str = typer.Argument(..., help="Component's name"),
    version: str = typer.Option(
        "1.0", "--version", "-v", help="Component's version"
    ),
    path: str = typer.Option("", "--path", "-p", help="Component's path"),
) -> None:
    try:
        component = Component(ctx.obj)
        component.create(name, version, path)
        abs_path = str(Path(path).resolve())
        console.print(
            f"Component {name} created successfully in {abs_path}", 
            style=success_style
        )

    except Exception as e:
        console.print(
            f"Error creating component: {str(e)}", style=error_style
        )
        typer.Exit(1)


@component_app.command()
def run(
    ctx: typer.Context,
    path: str = typer.Argument(..., help="Path to component source code"),
    input: str = typer.Option(None, "--input", "-i", help="Input Values"),
    component_id: str = typer.Option(
        None, "--component-id", "-id", help="Component's ID"
    ),
) -> None:
    try:
        component = Component(ctx.obj)
        console.print("Running component...", style=success_style)
        input = json.loads(input) if input else None
        component.run(path, input_parameters=input, component_id=component_id)
    except Exception as e:
        logger.exception(e)
        console.print(f"Error running component: {str(e)}", style=error_style)
        typer.Exit(1)

@component_app.command()
def upgrade(
    ctx: typer.Context,
    from_component_id: str = typer.Option(None, "--from-component-id", "-f", help="From Component's ID"),
    to_component_id: str = typer.Option(None, "--to-component-id", "-t", help="To Component's ID"),
) -> None:
    try:
        component = Component(ctx.obj)
        console.print("Upgrading component...", style=success_style)
        component.upgrade(from_component_id=from_component_id, to_component_id=to_component_id)
    except Exception as e:
        logger.exception(e)
        console.print(f"Error upgrading component: {str(e)}", style=error_style)
        typer.Exit(1)

@component_app.command()
def install_requirements(
    ctx: typer.Context,
    path: str = typer.Argument(..., help="Path to component source code"),
) -> None:
    try:
        component = Component(ctx.obj)
        console.print(
            "Installing component requirements...", style=success_style
        )
        component.install_requirements(path)
    except Exception as e:
        console.print(
            f"Error installing component requirements: {str(e)}",
            style=error_style,
        )
        typer.Exit(1)


@component_app.command()
def readme(
    ctx: typer.Context,
    path: str = typer.Argument(..., help="Path to component source code"),
    force: Optional[bool] = typer.Option(
        False,
        "--force",
        "-f",
        help="Delete if a Readme exists", 
    )
) -> None:
    try:
        component = Component(ctx.obj)
        console.print("Generating component README...", style=success_style)
        component.readme(path, force)
    except Exception as e:
        console.print(
            f"Error generating component README: {str(e)}",
            style=error_style
        )
        typer.Exit(1)






import typer
from rich.console import Console
from rich.table import Table

from cli.constants import error_style, success_style

workspace_app = typer.Typer(
    name="Splight CLI Workspace",
    add_completion=True,
    rich_markup_mode="rich",
)
console = Console()


@workspace_app.command()
def list(ctx: typer.Context) -> None:
    try:
        results = ctx.obj.workspace.list_workspaces()
        table = Table("WORKSPACES", show_lines=False, show_edge=False)
        _ = [
            table.add_row(item, style=success_style if "*" in item else None)
            for item in results
        ]
        console.print(table)
    except Exception as e:
        console.print(
            f"Error configuring Splight Hub: {str(e)}", style=error_style
        )
        typer.Exit(1)


@workspace_app.command()
def create(
    ctx: typer.Context,
    name: str = typer.Argument(..., help="The workspace's name"),
) -> None:
    try:
        ctx.obj.workspace.create_workspace(name)
        results = ctx.obj.workspace.list_workspaces()
        table = Table("WORKSPACES", show_lines=False, show_edge=False)
        _ = [
            table.add_row(item, style=success_style if "*" in item else None)
            for item in results
        ]
        console.print(table)
    except Exception as e:
        console.print(
            f"Error configuring Splight Hub: {str(e)}", style=error_style
        )
        typer.Exit(1)


@workspace_app.command()
def delete(
    ctx: typer.Context,
    name: str = typer.Argument(..., help="The workspace's name"),
) -> None:
    try:
        ctx.obj.workspace.delete_workspace(name)
        console.print(f"Deleted workspace {name}", style=success_style)
    except Exception as e:
        console.print(e, style=error_style)
        typer.Exit(1)


@workspace_app.command()
def select(
    ctx: typer.Context,
    name: str = typer.Argument(..., help="The workspace's name"),
) -> None:
    try:
        ctx.obj.workspace.select_workspace(name)
        console.print(f"Current workspace: {name}", style=success_style)
    except Exception as e:
        console.print(
            f"Error configuring Splight Hub: {str(e)}", style=error_style
        )
        typer.Exit(1)



import os
from datetime import datetime
from typing import Any, Dict, List, Optional, Type, Union

import pandas as pd
from pydantic import BaseModel
from rich.console import Console
from rich.table import Table
from splight_abstract import AbstractDatabaseClient, AbstractDatalakeClient
from splight_models import Component, DatalakeModel, SplightBaseModel

from cli.constants import success_style, warning_style

SplightModel = Type[SplightBaseModel]


class ResourceManagerException(Exception):
    pass


class DatalakeManagerException(Exception):
    pass


class QueryParam(BaseModel):
    value: Union[List[int], List[float], List[str], int, float, str]


class ResourceManager:
    def __init__(
        self,
        client: AbstractDatabaseClient,
        model: SplightModel,
    ):
        self._client = client
        self._model = model
        self._resource_name = model.__name__
        self._console = Console()

    def get(self, instance_id: str, exclude_fields: List[str] = None):
        exclude_fields = exclude_fields if exclude_fields is not None else []
        instance = self._client.get(self._model, id=instance_id, first=True)
        if not instance:
            raise ResourceManagerException(
                f"No {self._resources_name} found with ID = {instance_id}",
                style=warning_style,
            )

        name = instance.name if hasattr(instance, "name") else instance.title
        table = Table(
            title=f"{self._resource_name} = {name}", show_header=False
        )
        _ = [
            table.add_row(key, str(value))
            for key, value in instance.dict().items()
            if key not in exclude_fields
        ]
        self._console.print(table)

    def list(self, params: Dict[str, Any]):
        instances = self._client.get(
            resource_type=self._model,
            **params,
        )
        table = Table("", "ID", "Name")
        _ = [
            table.add_row(
                str(counter),
                item.id,
                item.name if hasattr(item, "name") else getattr(item, "title", "")
            )
            for counter, item in enumerate(instances)
        ]
        self._console.print(table)

    def create(self, data: Dict[str, Any]):
        instance = self._client.save(instance=self._model.parse_obj(data))
        table = Table(
            title=f"{self._resource_name} = {getattr(instance,'name', '')}", show_header=False
        )
        _ = [
            table.add_row(key, str(value))
            for key, value in instance.dict().items()
        ]
        self._console.print(table)

    def delete(self, instance_id: str):
        self._client.delete(resource_type=self._model, id=instance_id)
        self._console.print(
            f"{self._resource_name}={instance_id} deleted", style=warning_style
        )

    def download(self, instance_id: str, path: str):
        instance = self._client.get(self._model, id=instance_id, first=True)
        download = self._client.download(instance, decrypt=False)
        if not instance:
            raise ResourceManagerException(
                f"No {self._resources_name} found with ID = {instance_id}",
                style=warning_style,
            )
        if not path:
            path = instance.file
        with open(path, "wb+") as file:
            file.write(download.read())
        self._console.print(
            f"{self._resource_name}={instance_id} downloaded to {path}",
            style=warning_style,
        )

    @staticmethod
    def get_query_params(filters: Optional[List[str]]) -> Dict[str, Any]:
        params = {}
        filter_dict = {}
        for filter in filters:
            key, v = filter.split("=")
            if "__in" in key:
                value = filter_dict.get(key, [])
                value.append(v)
                filter_dict[key] = value
            else:
                filter_dict[key] = v

        params = {
            key: QueryParam(value=value).value
            for key, value in filter_dict.items()
        }
        return params


class DatalakeManager:
    def __init__(
        self,
        db_client: AbstractDatabaseClient,
        dl_client: AbstractDatalakeClient,
    ):
        self._db_client = db_client
        self._dl_client = dl_client
        self._console = Console()

    def list(self, skip, limit):
        instances = [{"id": "default", "name": "-"}]
        components = self._db_client.get(resource_type=Component)
        components = [component.dict() for component in components]
        instances.extend(components)
        instances = instances[
            skip: (limit + skip if limit is not None else None)
        ]
        table = Table("", "Name", "Component reference")
        _ = [
            table.add_row(
                str(counter),
                item.get("id"),
                item.get("name"),
            )
            for counter, item in enumerate(instances)
        ]
        self._console.print(table)

    def dump(self, collection, path, filters):
        if os.path.exists(path):
            raise Exception(f"File {path} already exists")
        if os.path.isdir(path):
            path = os.path.join(path, "splight_dump.csv")
        elif not path.endswith(".csv"):
            raise Exception("Only CSV files are supported")

        DatalakeModel.Meta.collection_name = collection
        dataframe = self._dl_client.get_dataframe(
            resource_type=DatalakeModel, **self._get_filters(filters)
        )
        dataframe.to_csv(path)
        self._console.print(
            f"Succesfully dumpped {collection} in {path}",
            style=success_style,
        )

    def load(self, collection, path):
        if not os.path.isfile(path):
            raise Exception("File not found")
        if not path.endswith(".csv"):
            raise Exception("Only CSV files are supported")

        dataframe = pd.read_csv(path)
        dataframe = dataframe.set_index("timestamp")
        DatalakeModel.Meta.collection_name = collection
        self._dl_client.save_dataframe(DatalakeModel, dataframe)
        self._console.print(
            f"Succesfully loaded {path} in {collection}",
            style=success_style,
        )

    @staticmethod
    def _to_list(key: str, elem: str):
        if "," not in elem and "__in" not in key:
            raise ValueError
        return list(elem.split(","))

    @staticmethod
    def _to_int(key: str, elem: str):
        if elem[0] == "-":
            can = elem[1:].isdigit()
        else:
            can = elem.isdigit()
        if not can:
            raise ValueError
        return int(elem)

    @staticmethod
    def _to_float(key: str, elem: str):
        try:
            return float(elem)
        except Exception:
            raise ValueError

    @staticmethod
    def _to_bool(key: str, elem: str):
        if elem == "True":
            return True
        elif elem == "False":
            return False
        else:
            raise ValueError

    @staticmethod
    def _to_date(key: str, elem: str):
        return datetime.strptime(elem, "%Y-%m-%dT%H:%M:%S%z")

    def _get_filters(self, filters):
        to_cast = [
            self._to_list,
            self._to_int,
            self._to_float,
            self._to_bool,
            self._to_date,
        ]
        parsed_filters = {}
        for key, value in filters.items():
            if key == "limit_":
                parsed_filters[key] = value
                continue
            for cast in to_cast:
                try:
                    value = cast(key, value)
                    break
                except ValueError:
                    pass

            parsed_filters[key] = value
        return parsed_filters



class ComponentAlreadyExists(Exception):
    def __init__(self, name: str, version: str):
        self._msg = f"Component {name}-{version} already exists in HUB"

    def __str__(self) -> str:
        return self._msg


class ComponentPullError(Exception):
    def __init__(self, name: str, version: str):
        self._msg = f"An error occurred pulling component {name}-{version}"

    def __str__(self) -> str:
        return self._msg


class ComponentPushError(Exception):
    def __init__(self, name: str, version: str):
        self._msg = f"An error occurred pushing component {name}-{version}"

    def __str__(self) -> str:
        return self._msg


class ComponentDirectoryAlreadyExists(Exception):
    def __init__(self, directory: str):
        self._msg = f"Directory with name {directory} already exists in path"

    def __str__(self) -> str:
        return self._msg



from typing import List, Optional

import typer
from rich.console import Console
from splight_models import File

from cli.constants import error_style
from cli.engine.manager import ResourceManager, ResourceManagerException

file_app = typer.Typer(
    name="Splight Engine File",
    add_completion=True,
    rich_markup_mode="rich",
    no_args_is_help=True,
)

console = Console()
MODEL = File


@file_app.command()
def list(
    ctx: typer.Context,
    filters: Optional[List[str]] = typer.Option(
        None,
        "--filter",
        "-f",
        help="Query param in the form key=value",
    )
):
    manager = ResourceManager(
        client=ctx.obj.framework.setup.DATABASE_CLIENT(),
        model=MODEL,
    )
    params = manager.get_query_params(filters)
    manager.list(params)


@file_app.command()
def get(
    ctx: typer.Context,
    instance_id: str = typer.Argument(..., help="The File's ID"),
    path: str = typer.Option(
        None, "--path", "-p", help="Path to save the file"
    ),
):
    manager = ResourceManager(
        client=ctx.obj.framework.setup.DATABASE_CLIENT(),
        model=MODEL,
    )
    try:
        manager.download(instance_id, path)
    except ResourceManagerException as exc:
        console.print(exc, style=error_style)


@file_app.command()
def create(
    ctx: typer.Context,
    path: str = typer.Argument(
        ..., help="Path to file to upload"
    ),
    description: str = typer.Option(
        None, "--description", "-d", help="Description of the file"
    ),
    encrypt: bool = typer.Option(
        False, "--encrypt", "-e", help="Encrypt the file"
    ),
):
    manager = ResourceManager(
        client=ctx.obj.framework.setup.DATABASE_CLIENT(),
        model=MODEL,
    )
    file = MODEL(
        description=description,
        encrypted=encrypt,
        file=path,
    )
    manager.create(file)


@file_app.command()
def delete(
    ctx: typer.Context,
    instance_id: str = typer.Argument(
        ..., help="The ID of the instance to be removed"
    ),
):
    manager = ResourceManager(
        client=ctx.obj.framework.setup.DATABASE_CLIENT(),
        model=MODEL,
    )
    manager.delete(instance_id)



from cli.constants import *
from cli.engine.manager import DatalakeManager
from cli.tests.test_generic import SplightCLITest
from cli.engine.datalake import datalake_app
from unittest.mock import patch
from fake_splight_lib.database import FakeDatabaseClient
from splight_models import Component

remote_collections = [
    Component(
        id='test_id',
        name='test component',
        version='1.0.0',
        description='',
        type=''
    ),
]


class TestList(SplightCLITest):
    def setUp(self):
        super().setUp()
        self.configure()

    @patch.object(FakeDatabaseClient, 'get', return_value=remote_collections)
    def test_list(self, mock_get):
        result = self.runner.invoke(datalake_app, 'list', obj=self.context, catch_exceptions=False)
        _stdout = result.output
        self.assertIn("Name", _stdout)
        self.assertIn("Component reference", _stdout)
        self.assertIn("default", _stdout)
        self.assertIn("-", _stdout)



import typer
from rich.console import Console
from splight_models import Component

from cli.constants import error_style
from cli.hub.component.hub_manager import HubComponentManager

component_app = typer.Typer(
    name="Splight Engine Component",
    add_completion=True,
    rich_markup_mode="rich",
    no_args_is_help=True,
)

console = Console()
MODEL = Component


@component_app.command()
def push(
    ctx: typer.Context,
    path: str = typer.Argument(..., help="Path to component source code"),
    force: bool = typer.Option(
        False,
        "--force",
        "-f",
        help="Overwrite existing component in Splight HUB",
    ),
):
    try:
        manager = HubComponentManager(
            client=ctx.obj.framework.setup.HUB_CLIENT()
        )
        manager.push(path, force=force)
    except Exception as exc:
        console.print(f"Error pushing component {exc}", style=error_style)
        raise typer.Exit(1)


@component_app.command()
def pull(
    ctx: typer.Context,
    name: str = typer.Argument(..., help="The component's name"),
    version: str = typer.Argument(..., help="The component's version"),
):
    try:
        manager = HubComponentManager(
            client=ctx.obj.framework.setup.HUB_CLIENT()
        )
        manager.pull(name=name, version=version)
    except Exception as exc:
        console.print(f"Error pulling component {exc}", style=error_style)
        raise typer.Exit(1)


@component_app.command()
def list(ctx: typer.Context):
    try:
        manager = HubComponentManager(
            client=ctx.obj.framework.setup.HUB_CLIENT()
        )
        manager.list_components()
    except Exception as exc:
        console.print(f"Error listing components {exc}", style=error_style)
        raise typer.Exit(1)


@component_app.command()
def versions(
    ctx: typer.Context, name: str = typer.Argument(..., help="Componet's name")
):
    try:
        manager = HubComponentManager(
            client=ctx.obj.framework.setup.HUB_CLIENT()
        )
        manager.versions(name=name)
    except Exception as exc:
        console.print(
            f"Error showing component's version {exc}", style=error_style
        )
        raise typer.Exit(1)



from pathlib import Path
from typing import List

from cli.constants import (
    CONFIG_FILE,
    DEFAULT_WORKSPACE,
    DEFAULT_WORKSPACE_NAME,
    DEFAULT_WORKSPACES,
)
from cli.settings import SplightCLIConfig, SplightCLISettings
from cli.utils.yaml import get_yaml_from_file, save_yaml_to_file


class WorkspaceDeleteError(Exception):
    def __init__(self, workspace: str):
        self._msg = (
            f"Workspace '{workspace}' is your active workspace\n\n"
            "You cannot delete the currently active workspace"
        )

    def __str__(self) -> str:
        return self._msg


class NotExistingWorkspace(Exception):
    def __init__(self, workspace: str):
        self._msg = f"Workspace {workspace} does not exist in your environment"

    def __str__(self) -> str:
        return self._msg


class WorkspaceManager:
    def __init__(self):
        self.config_file = CONFIG_FILE
        path = Path(self.config_file)
        path.parent.mkdir(parents=True, exist_ok=True)
        path.touch()
        self._config = self.__load_config()
        self._workspaces = self._config.workspaces
        self._current_workspace = self._config.current_workspace
        self._settings = self._workspaces[self._current_workspace]

    def __load_config(self):
        config = get_yaml_from_file(self.config_file)
        # Default values
        config["workspaces"] = config.get("workspaces", DEFAULT_WORKSPACES)
        workspace_name = config.get(
            "current_workspace", DEFAULT_WORKSPACE_NAME
        )
        if workspace_name not in config["workspaces"]:
            workspace_name = (
                DEFAULT_WORKSPACE_NAME
                if DEFAULT_WORKSPACE_NAME in config["workspaces"]
                else list(config["workspaces"].keys())[0]
            )
        config["current_workspace"] = workspace_name
        save_yaml_to_file(config, self.config_file)
        config = SplightCLIConfig.parse_obj(config)
        return config

    @property
    def settings(self) -> SplightCLISettings:
        return self._settings

    @property
    def current_workspace(self) -> str:
        return self._current_workspace

    def update_workspace(self, new_settings: SplightCLISettings):
        self._workspaces[self._current_workspace] = new_settings
        save_yaml_to_file(self._config.dict(), self.config_file)

    def select_workspace(self, workspace_name: str):
        if workspace_name not in self._workspaces:
            raise NotExistingWorkspace(workspace_name)
        self._config.current_workspace = workspace_name
        save_yaml_to_file(self._config.dict(), self.config_file)

    def list_workspaces(self) -> List[str]:
        return [
            f"{key}*" if key == self._current_workspace else f"{key}"
            for key in self._config.workspaces.keys()
        ]

    def delete_workspace(self, workspace_name: str):
        if workspace_name not in self._config.workspaces:
            raise Exception("Not a valid namespace name")

        if workspace_name == self._current_workspace:
            raise WorkspaceDeleteError(workspace_name)
        del self._config.workspaces[workspace_name]
        save_yaml_to_file(self._config.dict(), self.config_file)

    def create_workspace(self, workspace_name: str):
        if workspace_name in self._config.workspaces:
            raise Exception("Name already exists")
        self._config.workspaces.update({workspace_name: DEFAULT_WORKSPACE})
        self._config.current_workspace = workspace_name
        self._current_workspace = workspace_name
        save_yaml_to_file(self._config.dict(), self.config_file)



import requests
import json
from functools import wraps
from cli.constants import *
from requests.exceptions import ConnectionError

def http_request_validation(func):
    @wraps(func)
    def wrapper(*args, **kwargs):
        try:
            response = func(*args, **kwargs)
            if response.status_code >= 400:
                response = response.json()
                detail = response.get("detail", None)

                if detail in ["Invalid token.", "Authentication credentials were not provided.", "Incorrect authentication credentials."]:
                    raise Exception(f"{detail} Please check your Splight credentials. Use 'splightcli configure'")

                raise Exception(f"An unknown error ocurred while trying to reach Splight Hub API: {json.dumps(response)}")
            return response
        except ConnectionError:
            raise Exception(f"Splight Hub API is unreachable: Check out your configuration for Splight Hub API host. Use 'splightcli configure'")
    return wrapper

@http_request_validation
def api_get(*args, **kwargs):
    return requests.get(*args, **kwargs)

@http_request_validation
def api_post(*args, **kwargs):
    return requests.post(*args, **kwargs)

@http_request_validation
def api_delete(*args, **kwargs):
    return requests.delete(*args, **kwargs)



from typing import List

import typer
from rich.console import Console
from cli.constants import error_style
from cli.engine.manager import DatalakeManager, DatalakeManagerException

datalake_app = typer.Typer(
    name="Splight Engine Datalake",
    add_completion=True,
    rich_markup_mode="rich",
    no_args_is_help=True,
)
console = Console()


def _parse_filter_option(values):
    result = {}
    for value in values:
        k, v = value.split('=')
        result[k] = v
    return result


@datalake_app.command()
def list(
    ctx: typer.Context,
    skip: int = typer.Option(
        0, "--skip", "-s", help="Number of element to skip"
    ),
    limit: int = typer.Option(
        -1, "--limit", "-l", help="Limit the number of listed elements"
    ),
):
    manager = DatalakeManager(
        db_client=ctx.obj.framework.setup.DATABASE_CLIENT(),
        dl_client=ctx.obj.framework.setup.DATALAKE_CLIENT(),
    )
    manager.list(skip, limit)


@datalake_app.command()
def dump(
    ctx: typer.Context,
    collection: str = typer.Argument(
        ..., help="Collection name to dump"
    ),
    path: str = typer.Option(
        './dump.csv', "--path", "-p", help="Path name to dump"
    ),
    filter: List[str] = typer.Option(
        None, "--filter", "-f", help="Filter to apply"
    ),
):
    filters = _parse_filter_option(filter)
    manager = DatalakeManager(
        db_client=ctx.obj.framework.setup.DATABASE_CLIENT(),
        dl_client=ctx.obj.framework.setup.DATALAKE_CLIENT(),
    )
    try:
        manager.dump(
            collection=collection,
            path=path,
            filters=filters
        )
    except DatalakeManagerException as exc:
        console.print(exc, style=error_style)


@datalake_app.command()
def load(
    ctx: typer.Context,
    collection: str = typer.Argument(..., help="Collection name to load"),
    path: str = typer.Option(
        ..., "--path", "-p", help="Path to file to load"
    ),
):
    manager = DatalakeManager(
        db_client=ctx.obj.framework.setup.DATABASE_CLIENT(),
        dl_client=ctx.obj.framework.setup.DATALAKE_CLIENT(),
    )
    try:
        manager.load(
            collection=collection,
            path=path
        )
    except DatalakeManagerException as exc:
        console.print(exc, style=error_style)



from setuptools import setup, find_packages
from version import __version__

with open('requirements.txt') as fp:
    install_requires = fp.readlines()

dependency_links = [
    # External repositories different from pypi
]

setup(
    name='splight-cli',
    version=__version__,
    author='Splight',
    author_email='factory@splight-ae.com',
    description='CLI tool to build and run Splight components',
    py_modules=['splight'],
    install_requires=install_requires,
    packages=find_packages(),
    package_data={
        'cli': [
            'component/templates/*',
            'datalake/dump_example.csv',
        ],
    },
    include_package_data=True,
    dependency_links=dependency_links,
    entry_points={
        'console_scripts': [
            'splight = cli:cli.app',
        ],
    },
)



# import click
import typer


class Printer:
    DEFAULT_COLOR = "white"

    @staticmethod
    def print_dict(items, headers, items_colors=[]):
        format_str = "{:<50}" * len(headers)
        typer.echo(format_str.format(*[h.upper() for h in headers]))
        for index, item in enumerate(items):
            try:
                fg = items_colors[index]
            except IndexError:
                fg = Printer.DEFAULT_COLOR
            typer.echo(
                format_str.format(*[item.get(key) for key in headers]),
                color=fg,
            )

    @staticmethod
    def print_list(items, header=None, items_colors=[]):
        if header is not None:
            typer.echo(header.upper())
        for index, item in enumerate(items):
            try:
                fg = items_colors[index]
            except IndexError:
                fg = Printer.DEFAULT_COLOR
            typer.echo(item, color=fg)

    @staticmethod
    def print_value(item, color=None):
        fg = color if color is not None else Printer.DEFAULT_COLOR
        typer.echo(item, color=fg)



from splight_lib import logging
from splight_models import Deployment
from cli.utils import *
from cli.constants import DEFAULT_NAMESPACE

logger = logging.getLogger()


class DeploymentHandler:

    def __init__(self, context):
        self.context = context
        self.namespace = DEFAULT_NAMESPACE

    @property
    def client(self):
        return self.context.framework.setup.DEPLOYMENT_CLIENT(self.namespace)

    def list(self):
        return [
            i.dict()
            for i in self.client.get(Deployment)
        ]


import os
from unittest.mock import patch, call
from cli.component.component import Component
from cli.tests.test_generic import SplightCLITest
from cli.constants import *
import subprocess

class TestInitialize(SplightCLITest):
    
    def test_initialize(self):
        self.component = Component(self.context)
        initialization_file_path = os.path.join(self.path, INIT_FILE)
        lines = []

        with open(initialization_file_path) as f:
            for line in f:
                line = line.strip()
                if line.startswith("#") or line == "":
                    continue
                lines.append(line)

        run_commands = [" ".join(line.split(" ")[1:]) for line in lines if line.startswith("RUN")]

        with patch.object(subprocess, "run") as runs:
            self.component.install_requirements(self.path)
            commands = run_commands
            calls = []
            for command in commands:
                calls.append(call(command, check=True, cwd=self.path, shell=True))
            runs.assert_has_calls(calls)


# import click
import typer

# from functools import wraps
from rich.console import Console

from cli.constants import error_style
from cli.context.framework import FrameworkManager
from cli.context.workspace import WorkspaceManager

console = Console()


class Context:
    def __init__(self):
        self.__workspace = WorkspaceManager()
        self.__framework = FrameworkManager()

    @property
    def framework(self):
        self.__framework.configure(self.workspace.settings.dict())
        return self.__framework

    @property
    def workspace(self):
        return self.__workspace

    def __repr__(self):
        return f"<Context {self.__workspace}>"


# pass_context = click.make_pass_decorator(Context)


def check_credentials(ctx: typer.Context):
    settings = ctx.obj.workspace.settings

    access_id = settings.SPLIGHT_ACCESS_ID
    secret_key = settings.SPLIGHT_SECRET_KEY
    if not access_id or not secret_key:
        console.print(
            'Please set you Splight credentials with "splight configure"',
            style=error_style,
        )
        raise typer.Exit(1)


# def needs_credentials(f):
#     @wraps(f)
#     def wrapper(ctx: typer.Context, *args, **kwargs):
#         __import__('ipdb').set_trace()
#         access_id = ctx.workspace.settings.SPLIGHT_ACCESS_ID
#         secret_key = ctx.workspace.settings.SPLIGHT_SECRET_KEY
#         if access_id is None or secret_key is None:
#             click.secho(
#                 "Please set your Splight credentials. Use \"splightcli configure\"",
#                 fg="red"
#             )
#             exit(1)
#         return f(ctx, *args, **kwargs)
#     return wrapper



import random
from typing import TypeVar
from splight_lib.component import AbstractComponent
from splight_lib.execution import Task
from splight_lib import logging

# Logging tool
logger = logging.getLogger()

# Custom Types
## NOTE: In case you want to create new instances of this Class
## You can find this model in self.custom_model.MyAsset inside the Main class
MyAsset = TypeVar('MyAsset')


class Main(AbstractComponent):

    # Init
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.my_assets = {
            item.id: item
            for item in self.database_client.get(self.custom_types.MyAsset)
        }
        logger.info(f"Starting randomizer in range {self.input.min} - {self.input.max}")

    # Starting point
    def start(self):
        self.execution_client.start(
            Task(handler=self._give_a_random_number, args=(self.input.min, self.input.max), period=self.input.period)
        )
        self.execution_client.start(
            Task(handler=self._list_assets, args=(), period=self.input.period)
        )

    # Periodic Tasks
    def _give_a_random_number(self, min, max):
        chosen_number = random.randint(min, max)
        logger.info(f"Random number: {chosen_number}")
        out = self.output.Value(value=chosen_number)
        self.datalake_client.save(instances=[out])

    def _list_assets(self):
        logger.info(f"List of myassets: List[MyAsset] {self.my_assets}")

    # Bindings
    def handle_myasset_create(self, my_asset: MyAsset):
        self.my_assets[my_asset.id] = my_asset
        logger.info(f"CREATED MyAsset: {my_asset}")

    def handle_myasset_delete(self, my_asset: MyAsset):
        try:
            logger.info(f"DELETED MyAsset: {my_asset}")
            del self.my_assets[my_asset.id]
        except KeyError:
            pass

    # Commands
    def command_myasset_print(self, myasset: MyAsset):
        logger.info(f"PRINTED MyAsset: {myasset}")
        return myasset.dict()


from typing import Callable, TypeVar

from splight_lib import logging
from splight_lib.settings import setup

logger = logging.getLogger()
RetType = TypeVar("RetType")
OriginalFunc = Callable[..., RetType]
DecoratedFunc = Callable[..., RetType]


class MissingConfiguration(Exception):
    pass


class FrameworkManager:
    @property
    def setup(self):
        return setup

    def configure(self, environment):
        empty_keys = [
            key for key, value in environment.items() if value is None
        ]
        if empty_keys:
            raise MissingConfiguration(
                f"Please run `splightcli configure` and set keys {empty_keys}"
            )
        setup.configure(environment)



import os
import shutil
from cli.component.loaders import ComponentLoader
from cli.component.component import Component
from cli.tests.test_generic import SplightCLITest


class TestCreate(SplightCLITest):
    def setUp(self):
        super().setUp()
        self.component = Component(self.context)

    def tearDown(self) -> None:
        shutil.rmtree(self.custompath)
        super().tearDown()

    def test_create_on_custom_path(self):
        self.component.create(self.name, self.version, self.custompath)
        for filename in ComponentLoader.REQUIRED_FILES:
            self.assertTrue(
                os.path.exists(os.path.join(self.custompath, filename))
            )

    def test_create_already_exists(self):
        os.makedirs(self.custompath)
        self.component.create(self.name, self.version, self.custompath)

        for filename in ComponentLoader.REQUIRED_FILES:
            self.assertTrue(
                os.path.exists(os.path.join(self.custompath, filename))
            )



from cli.utils.template import *
from cli.utils.validation import *
from cli.utils.json import *
from cli.utils.api_requests import *
from cli.utils.yaml import *
from cli.utils.input import *
from cli.utils.pprint import *
from cli.utils.uuid import *